(* PBM Reader - reads PBM files and displays them on the screen. *)
(* Version 3.2 *)
program PbmReader;
(*$IKERNEL.INC*)
(*$IGRAPHLIB.INC*)
(*$IINPUT.INC*)
(*$IBOX.INC*)
(*$ISCREEN.INC*)
(*$IPBMIN.INC*)

const
  SearchFirst: Integer = $11;
  SearchNext : Integer = $12;
  SetDMA     : Integer = $1A;
  MaxFiles = 100;
label
  byebye;
var
  Drive:        string(.2.);
  FileName:     string(.14.);
  DiashowDelay: integer; (* Delay between diashow pictures in seconds *)
  err:          ErrorCode;
  invert:       boolean;
  ch:           char;
  
  
(* Display diashow *)
procedure Diashow;
label
  ExitDiashow;
var
  run:       Boolean;
  i, j:      Integer;
  drive:     string(.2.);
  FileNames: array(.1 .. MaxFiles.) of string(.14.);
  fileCount: Integer;
  ch:        Char;

  (* Reads VEC filenames into the FileNames array.
     Returns the number of files found. *)
  function ReadDirectory: Integer;
  var
    error, loop, start: Integer;
    FCB: array(.0 ..  26.) of Byte absolute $0060;
    DMA: array(.0 .. 266.) of Byte;
    idx: Integer;

    (* Copies a filename from the DMA array to the FileName variable. *)
    procedure DMAtoFileName(start: Integer);
    var
      loop: Integer;
      ch  : Char;
    begin
      FileName := '';
      for loop := start + 1 to start + 8 do begin (* copy filename *)
        ch := Char( Mem(.Addr(DMA) + loop.) );
        if ch <> ' ' then begin
          FileName := FileName + ch;
        end;
      end;
      FileName := FileName + '.';
      for loop := start + 9 to start + 11 do begin (* copy extension *)
        ch := Char( Mem(.Addr(DMA) + loop.) );
        if ch <> ' ' then begin
          FileName := FileName + ch;
        end;
      end;
    end; (* DMAtoFileName *)

  begin
    error := BDos(SetDMA, Addr(DMA));
    FCB(.0.) := 0;
    (* filter: files with extension mask PBM only *)
    for loop := 1 to 8 do
      FCB(.loop.) := ord('?');
    FCB(. 9.) := ord('P');
    FCB(.10.) := ord('B');
    FCB(.11.) := ord('M');

    idx := 1;
    error:= BDos(SearchFirst, Addr(FCB));
    if error <> 255 then begin
      start := error * 32;
      DMAtoFileName(start);
      FileNames(.idx.) := FileName;
      idx := succ(idx);
    end;
    repeat
      error := BDos(SearchNext);
      start := error * 32;
      if error <> 255 then begin
        DMAtoFileName(start);
        FileNames(.idx.) := FileName;
        idx := succ(idx);
        if idx > MaxFiles then
          error := 255; (* exit loop *)
      end
    until error = 255;

    ReadDirectory := pred(idx);
  end; (* ReadDirectory *)

begin (* main procedure *)
  GraphInit;
  HideCursor;
  
  fileCount := ReadDirectory;
  if fileCount > MaxFiles then begin
    fileCount := MaxFiles;
  end;
  run := true;

  while run do begin
    for i := 1 to fileCount do begin
      FileName := FileNames(.i.);
      if Length(Drive) = 2 then FileName := Drive + FileName;
      err := LoadScreenFromPBM(FileName, invert);
      
      for j := 1 to DiashowDelay do begin
        Delay(1000);
        if KeyPressed then begin
          Read(kbd, ch);
          goto ExitDiashow;
        end;
      end; (* for j *)
    end; (* for i *)
  end; (* while *)
ExitDiashow:
end; (* Diashow *)


(* Inverts the screen in memory, i.e. without reading the file again. *)
procedure InvertInMemory;
var
  row: byte;
  i: integer;
begin
  for row := 0 to 31 do begin
    GetScreenRow(row);
    for i := 0 to 719 do begin
      GX_Buffer(.i.) := GX_Buffer(.i.) xor $FF;
    end;
    PutScreenRow(row);
  end;
end; (* InvertInMemory *)


(* Main program *)
begin
  ClrScr;
  invert := false;
  DiashowDelay := 5;
  
  (* get the name of the PBM file *)
  if ParamCount > 0 then begin
    if ParamStr(1) = '-DIASHOW' then begin
      if ParamCount > 1 then begin
        if Length(ParamStr(2)) = 2 then begin
          Drive := ParamStr(2);
          (* check if drive is one of A: to Z: *)
          if (Drive(.1.) < 'A' ) or (Drive(.1.) > 'Z') or (Drive(.2.) <> ':') then
            Drive := ''; (* drive spec wrong - use current drive *)
        end
        else
          Drive := ''; (* no drive provided - use current drive *)
      end;
      Diashow;
      goto byebye;
    end
    else begin
      FileName := ParamStr(1); (* first commandline parameter treated as filename *)
      if ParamCount > 1 then begin
        if ParamStr(2) = '-INVERT' then invert := true;
      end;
    end;
  end
  else begin
    DrawBox(1, 1, 90, 3);
    GotoXY(28, 2);
    Write('PBM file reader V3.2 by Bernd Bock');
    DrawBox(1, 4, 90, 3);
    GotoXY(3, 5);
    Write('Enter name of PBM file: ');
    FileName := InputFileName;
    GotoXY(1, 7);
    if Length(FileName) = 0 then goto byebye; (* abort if no filename entered *)
    Write('Invert the picture? (Y/N) ');
    ch := InputChar;
    if (ch = 'Y') or (ch = 'y') then invert := true;
  end;
  
  GraphInit;
  HideCursor;

  err := LoadScreenFromPBM(FileName, invert);
  if err <> ErrOk then begin
    WriteLn;
    case err of
      ErrFileNotFound:       WriteLn('File not found: ', FileName);
      ErrFormatNotSupported: WriteLn('Format not supported: only 720x256 or 720x512 binary - PBM type 4')
      else                   WriteLn('Unknown error!');
    end;
    WriteKeyMessage;
    WaitForKey;
  end
  else
    repeat
      ch := InputCharNoEcho;
      if ch = 'i' then ch := 'I';            (* 'I' input always uppercase *)
      if ch = 'I' then InvertInMemory;
    until ch <> 'I';
  
byebye:  
  Write(chr(27), '0'); (* switch off status bar *)
  ClrScr;              (* now clears the status bar, too *)
  Write(chr(27), '1'); (* switch on status bar *)
  ShowCursor;
end.
